@@ -1,7 +1,6 @@
 import numpy as np
 import os
 import math
-from .full_model import *
 from .utils import *
 from .mfp2DSym import *
 from .mg2DSym import *
@@ -18,33 +17,32 @@ comm = MPI.COMM_WORLD
 
 def Material(**argv):
 
-   model = argv['model']
+  if 'custom' in argv.keys():
+    data = argv['custom'](argv)
+  else:  
+
+   model = argv.setdefault('model','rta3D')
 
    #set up database
-   source = argv.setdefault('source','local')
+   source = argv.setdefault('source','database')
    if source == 'database':
     if comm.rank == 0:
-        filename = os.path.dirname(os.path.dirname(os.path.realpath(__file__))) + \
+        if 'temperature' in argv.keys():
+         filename = os.path.dirname(os.path.dirname(os.path.realpath(__file__))) + \
                   '/openbte/materials/rta_' +  argv['filename'] + \
                   '_' + str(argv['temperature']) 
+        else:          
+         filename = os.path.dirname(os.path.dirname(os.path.realpath(__file__))) + \
+                  '/openbte/materials/' +  argv['filename']  
 
         argv['filename'] = filename
 
-   if source == 'unlisted':
-    if comm.rank == 0:
-      download_file(argv['file_id'],'rta.npz')
+
 
    if model == 'rta2DSym':
      if comm.rank == 0:
       data = rta2DSym(**argv)
 
-   elif model == 'full':
-     if comm.rank == 0:
-      data = full(**argv)
-
-   elif model == 'mfp2DSym':
-      data = mfp2DSym(**argv)
-
    elif model == 'fourier':
       
       kappa = np.eye(3)
@@ -62,15 +60,11 @@ def Material(**argv):
    elif model == 'gray2DSym':
       data = gray2DSym(**argv)
 
-   elif model == 'mg2DSym':
-     if comm.rank == 0:
-      data = mg2DSym(**argv)
-
    elif model == 'gray2D':
       data = gray2D(**argv)
 
-   #elif model == 'gray3D':
-   #   data = mfp3D(**argv)
+    #elif model == 'gray3D':
+    #   data = mfp3D(**argv)
    
    elif model == 'mfp3D':
       data = mfp3D(**argv)
@@ -84,11 +78,11 @@ def Material(**argv):
       quit()
 
 
-   if argv.setdefault('save',True):
+  if argv.setdefault('save',True):
      if comm.rank == 0:
          save_data(argv.setdefault('output_filename','material'),data)   
 
-   if comm.rank == 0:
+  if comm.rank == 0:
     return data
 
  
